@@ -1,28 +1,20 @@
 /*  Lzlib - Compression library for the lzip format
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2018 Antonio Diaz Diaz.
 
-    This library is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software. Redistribution and use in source and
+    binary forms, with or without modification, are permitted provided
+    that the following conditions are met:
+
+    1. Redistributions of source code must retain the above copyright
+    notice, this list of conditions and the following disclaimer.
+
+    2. Redistributions in binary form must reproduce the above copyright
+    notice, this list of conditions and the following disclaimer in the
+    documentation and/or other materials provided with the distribution.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this library.  If not, see <http://www.gnu.org/licenses/>.
-
-    As a special exception, you may use this file as part of a free
-    software library without restriction.  Specifically, if other files
-    instantiate templates or use macros or inline functions from this
-    file, or you compile this file and link it with other files to
-    produce an executable, this file does not by itself cause the
-    resulting executable to be covered by the GNU General Public
-    License.  This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General
-    Public License.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 
 #include <stdbool.h>
@@ -35,33 +27,33 @@
 #include "cbuffer.c"
 #include "decoder.h"
 #include "decoder.c"
+#include "encoder_base.h"
+#include "encoder_base.c"
 #include "encoder.h"
 #include "encoder.c"
+#include "fast_encoder.h"
+#include "fast_encoder.c"
 
 
 struct LZ_Encoder
   {
   unsigned long long partial_in_size;
   unsigned long long partial_out_size;
-  struct Matchfinder * matchfinder;
-  struct LZ_encoder * lz_encoder;
+  struct LZ_encoder_base * lz_encoder_base;	/* these 3 pointers make a */
+  struct LZ_encoder * lz_encoder;		/* polymorphic encoder */
+  struct FLZ_encoder * flz_encoder;
   enum LZ_Errno lz_errno;
-  int flush_pending;
-  File_header member_header;
   bool fatal;
   };
 
-static void LZ_Encoder_init( struct LZ_Encoder * const e,
-                             const File_header header )
+static void LZ_Encoder_init( struct LZ_Encoder * const e )
   {
-  int i;
   e->partial_in_size = 0;
   e->partial_out_size = 0;
-  e->matchfinder = 0;
+  e->lz_encoder_base = 0;
   e->lz_encoder = 0;
+  e->flz_encoder = 0;
   e->lz_errno = LZ_ok;
-  e->flush_pending = 0;
-  for( i = 0; i < Fh_size; ++i ) e->member_header[i] = header[i];
   e->fatal = false;
   }
 
@@ -75,6 +67,7 @@ struct LZ_Decoder
   enum LZ_Errno lz_errno;
   File_header member_header;		/* header of current member */
   bool fatal;
+  bool first_header;			/* true until first header is read */
   bool seeking;
   };
 
@@ -88,6 +81,7 @@ static void LZ_Decoder_init( struct LZ_Decoder * const d )
   d->lz_errno = LZ_ok;
   for( i = 0; i < Fh_size; ++i ) d->member_header[i] = 0;
   d->fatal = false;
+  d->first_header = true;
   d->seeking = false;
   }
 
@@ -95,7 +89,8 @@ static void LZ_Decoder_init( struct LZ_Decoder * const d )
 static bool verify_encoder( struct LZ_Encoder * const e )
   {
   if( !e ) return false;
-  if( !e->matchfinder || !e->lz_encoder )
+  if( !e->lz_encoder_base || ( !e->lz_encoder && !e->flz_encoder ) ||
+      ( e->lz_encoder && e->flz_encoder ) )
     { e->lz_errno = LZ_bad_argument; return false; }
   return true;
   }
@@ -120,15 +115,15 @@ const char * LZ_strerror( const enum LZ_Errno lz_errno )
   switch( lz_errno )
     {
     case LZ_ok            : return "ok";
-    case LZ_bad_argument  : return "bad argument";
-    case LZ_mem_error     : return "not enough memory";
-    case LZ_sequence_error: return "sequence error";
-    case LZ_header_error  : return "header error";
-    case LZ_unexpected_eof: return "unexpected eof";
-    case LZ_data_error    : return "data error";
-    case LZ_library_error : return "library error";
+    case LZ_bad_argument  : return "Bad argument";
+    case LZ_mem_error     : return "Not enough memory";
+    case LZ_sequence_error: return "Sequence error";
+    case LZ_header_error  : return "Header error";
+    case LZ_unexpected_eof: return "Unexpected eof";
+    case LZ_data_error    : return "Data error";
+    case LZ_library_error : return "Library error";
     }
-  return "invalid error code";
+  return "Invalid error code";
   }
 
 
@@ -147,35 +142,31 @@ struct LZ_Encoder * LZ_compress_open( const int dictionary_size,
                                       const unsigned long long member_size )
   {
   File_header header;
-  const bool error = ( !Fh_set_dictionary_size( header, dictionary_size ) ||
-                       match_len_limit < min_match_len_limit ||
-                       match_len_limit > max_match_len ||
-                       member_size < min_dictionary_size );
-
   struct LZ_Encoder * const e =
     (struct LZ_Encoder *)malloc( sizeof (struct LZ_Encoder) );
   if( !e ) return 0;
-  Fh_set_magic( header );
-  LZ_Encoder_init( e, header );
-  if( error ) e->lz_errno = LZ_bad_argument;
+  LZ_Encoder_init( e );
+  if( !Fh_set_dictionary_size( header, dictionary_size ) ||
+      match_len_limit < min_match_len_limit ||
+      match_len_limit > max_match_len ||
+      member_size < min_dictionary_size )
+    e->lz_errno = LZ_bad_argument;
   else
     {
-    e->matchfinder = (struct Matchfinder *)malloc( sizeof (struct Matchfinder) );
-    if( e->matchfinder )
+    if( dictionary_size == 65535 && match_len_limit == 16 )
+      {
+      e->flz_encoder = (struct FLZ_encoder *)malloc( sizeof (struct FLZ_encoder) );
+      if( e->flz_encoder && FLZe_init( e->flz_encoder, member_size ) )
+        { e->lz_encoder_base = &e->flz_encoder->eb; return e; }
+      free( e->flz_encoder ); e->flz_encoder = 0;
+      }
+    else
       {
       e->lz_encoder = (struct LZ_encoder *)malloc( sizeof (struct LZ_encoder) );
-      if( e->lz_encoder )
-        {
-        if( Mf_init( e->matchfinder,
-                     Fh_get_dictionary_size( header ), match_len_limit ) )
-          {
-          if( LZe_init( e->lz_encoder, e->matchfinder, header, member_size ) )
-            return e;
-          Mf_free( e->matchfinder );
-          }
-        free( e->lz_encoder ); e->lz_encoder = 0;
-        }
-      free( e->matchfinder ); e->matchfinder = 0;
+      if( e->lz_encoder && LZe_init( e->lz_encoder, Fh_get_dictionary_size( header ),
+                                     match_len_limit, member_size ) )
+        { e->lz_encoder_base = &e->lz_encoder->eb; return e; }
+      free( e->lz_encoder ); e->lz_encoder = 0;
       }
     e->lz_errno = LZ_mem_error;
     }
@@ -187,10 +178,9 @@ struct LZ_Encoder * LZ_compress_open( const int dictionary_size,
 int LZ_compress_close( struct LZ_Encoder * const e )
   {
   if( !e ) return -1;
-  if( e->lz_encoder )
-    { LZe_free( e->lz_encoder ); free( e->lz_encoder ); }
-  if( e->matchfinder )
-   { Mf_free( e->matchfinder ); free( e->matchfinder ); }
+  if( e->lz_encoder_base )
+    { LZeb_free( e->lz_encoder_base );
+      free( e->lz_encoder ); free( e->flz_encoder ); }
   free( e );
   return 0;
   }
@@ -199,13 +189,17 @@ int LZ_compress_close( struct LZ_Encoder * const e )
 int LZ_compress_finish( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) || e->fatal ) return -1;
-  Mf_finish( e->matchfinder );
-  e->flush_pending = 0;
+  Mb_finish( &e->lz_encoder_base->mb );
   /* if (open --> write --> finish) use same dictionary size as lzip. */
   /* this does not save any memory. */
-  if( Mf_data_position( e->matchfinder ) == 0 &&
-      LZ_compress_total_out_size( e ) == Fh_size )
-    Mf_adjust_distionary_size( e->matchfinder );
+  if( Mb_data_position( &e->lz_encoder_base->mb ) == 0 &&
+      Re_member_position( &e->lz_encoder_base->renc ) == Fh_size )
+    {
+    Mb_adjust_dictionary_size( &e->lz_encoder_base->mb );
+    Fh_set_dictionary_size( e->lz_encoder_base->renc.header,
+                            e->lz_encoder_base->mb.dictionary_size );
+    e->lz_encoder_base->renc.cb.buffer[5] = e->lz_encoder_base->renc.header[5];
+    }
   return 0;
   }
 
@@ -214,22 +208,16 @@ int LZ_compress_restart_member( struct LZ_Encoder * const e,
                                 const unsigned long long member_size )
   {
   if( !verify_encoder( e ) || e->fatal ) return -1;
-  if( !LZe_member_finished( e->lz_encoder ) )
+  if( !LZeb_member_finished( e->lz_encoder_base ) )
     { e->lz_errno = LZ_sequence_error; return -1; }
   if( member_size < min_dictionary_size )
     { e->lz_errno = LZ_bad_argument; return -1; }
 
-  e->partial_in_size += Mf_data_position( e->matchfinder );
-  e->partial_out_size += Re_member_position( &e->lz_encoder->renc );
-  Mf_reset( e->matchfinder );
+  e->partial_in_size += Mb_data_position( &e->lz_encoder_base->mb );
+  e->partial_out_size += Re_member_position( &e->lz_encoder_base->renc );
 
-  LZe_free( e->lz_encoder );
-  if( !LZe_init( e->lz_encoder, e->matchfinder, e->member_header, member_size ) )
-    {
-    LZe_free( e->lz_encoder ); free( e->lz_encoder ); e->lz_encoder = 0;
-    e->lz_errno = LZ_mem_error; e->fatal = true;
-    return -1;
-    }
+  if( e->lz_encoder ) LZe_reset( e->lz_encoder, member_size );
+  else FLZe_reset( e->flz_encoder, member_size );
   e->lz_errno = LZ_ok;
   return 0;
   }
@@ -238,15 +226,8 @@ int LZ_compress_restart_member( struct LZ_Encoder * const e,
 int LZ_compress_sync_flush( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) || e->fatal ) return -1;
-  if( e->flush_pending <= 0 && !Mf_flushing_or_end( e->matchfinder ) )
-    {
-    e->flush_pending = 2;	/* 2 consecutive markers guarantee decoding */
-    e->matchfinder->flushing = true;
-    if( !LZe_encode_member( e->lz_encoder ) )
-      { e->lz_errno = LZ_library_error; e->fatal = true; return -1; }
-    while( e->flush_pending > 0 && LZe_sync_flush( e->lz_encoder ) )
-      { if( --e->flush_pending <= 0 ) e->matchfinder->flushing = false; }
-    }
+  if( !Mb_flushing_or_end( &e->lz_encoder_base->mb ) )
+    e->lz_encoder_base->mb.flushing = true;
   return 0;
   }
 
@@ -254,12 +235,24 @@ int LZ_compress_sync_flush( struct LZ_Encoder * const e )
 int LZ_compress_read( struct LZ_Encoder * const e,
                       uint8_t * const buffer, const int size )
   {
+  int out_size = 0;
   if( !verify_encoder( e ) || e->fatal ) return -1;
-  if( !LZe_encode_member( e->lz_encoder ) )
-    { e->lz_errno = LZ_library_error; e->fatal = true; return -1; }
-  while( e->flush_pending > 0 && LZe_sync_flush( e->lz_encoder ) )
-    { if( --e->flush_pending <= 0 ) e->matchfinder->flushing = false; }
-  return Re_read_data( &e->lz_encoder->renc, buffer, size );
+  if( size < 0 ) return 0;
+  do {
+    if( ( e->flz_encoder && !FLZe_encode_member( e->flz_encoder ) ) ||
+        ( e->lz_encoder && !LZe_encode_member( e->lz_encoder ) ) )
+      { e->lz_errno = LZ_library_error; e->fatal = true; return -1; }
+    if( e->lz_encoder_base->mb.flushing &&
+        Mb_available_bytes( &e->lz_encoder_base->mb ) <= 0 &&
+        LZeb_sync_flush( e->lz_encoder_base ) )
+      e->lz_encoder_base->mb.flushing = false;
+    out_size += Re_read_data( &e->lz_encoder_base->renc,
+                              buffer + out_size, size - out_size );
+    }
+  while( e->lz_encoder_base->mb.flushing && out_size < size &&
+         Mb_enough_available_bytes( &e->lz_encoder_base->mb ) &&
+         Re_enough_free_bytes( &e->lz_encoder_base->renc ) );
+  return out_size;
   }
 
 
@@ -267,16 +260,14 @@ int LZ_compress_write( struct LZ_Encoder * const e,
                        const uint8_t * const buffer, const int size )
   {
   if( !verify_encoder( e ) || e->fatal ) return -1;
-  if( e->flush_pending > 0 || size < 0 ) return 0;
-  return Mf_write_data( e->matchfinder, buffer, size );
+  return Mb_write_data( &e->lz_encoder_base->mb, buffer, size );
   }
 
 
 int LZ_compress_write_size( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) || e->fatal ) return -1;
-  if( e->flush_pending > 0 ) return 0;
-  return Mf_free_bytes( e->matchfinder );
+  return Mb_free_bytes( &e->lz_encoder_base->mb );
   }
 
 
@@ -290,43 +281,43 @@ enum LZ_Errno LZ_compress_errno( struct LZ_Encoder * const e )
 int LZ_compress_finished( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) ) return -1;
-  return ( e->flush_pending <= 0 && Mf_finished( e->matchfinder ) &&
-           LZe_member_finished( e->lz_encoder ) );
+  return ( Mb_data_finished( &e->lz_encoder_base->mb ) &&
+           LZeb_member_finished( e->lz_encoder_base ) );
   }
 
 
 int LZ_compress_member_finished( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) ) return -1;
-  return LZe_member_finished( e->lz_encoder );
+  return LZeb_member_finished( e->lz_encoder_base );
   }
 
 
 unsigned long long LZ_compress_data_position( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) ) return 0;
-  return Mf_data_position( e->matchfinder );
+  return Mb_data_position( &e->lz_encoder_base->mb );
   }
 
 
 unsigned long long LZ_compress_member_position( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) ) return 0;
-  return Re_member_position( &e->lz_encoder->renc );
+  return Re_member_position( &e->lz_encoder_base->renc );
   }
 
 
 unsigned long long LZ_compress_total_in_size( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) ) return 0;
-  return e->partial_in_size + Mf_data_position( e->matchfinder );
+  return e->partial_in_size + Mb_data_position( &e->lz_encoder_base->mb );
   }
 
 
 unsigned long long LZ_compress_total_out_size( struct LZ_Encoder * const e )
   {
   if( !verify_encoder( e ) ) return 0;
-  return e->partial_out_size + Re_member_position( &e->lz_encoder->renc );
+  return e->partial_out_size + Re_member_position( &e->lz_encoder_base->renc );
   }
 
 
@@ -363,7 +354,8 @@ int LZ_decompress_close( struct LZ_Decoder * const d )
 int LZ_decompress_finish( struct LZ_Decoder * const d )
   {
   if( !verify_decoder( d ) || d->fatal ) return -1;
-  if( d->seeking ) { d->seeking = false; Rd_purge( d->rdec ); }
+  if( d->seeking )
+    { d->seeking = false; d->partial_in_size += Rd_purge( d->rdec ); }
   else Rd_finish( d->rdec );
   return 0;
   }
@@ -379,6 +371,7 @@ int LZ_decompress_reset( struct LZ_Decoder * const d )
   Rd_reset( d->rdec );
   d->lz_errno = LZ_ok;
   d->fatal = false;
+  d->first_header = true;
   d->seeking = false;
   return 0;
   }
@@ -386,15 +379,17 @@ int LZ_decompress_reset( struct LZ_Decoder * const d )
 
 int LZ_decompress_sync_to_member( struct LZ_Decoder * const d )
   {
+  unsigned skipped = 0;
   if( !verify_decoder( d ) ) return -1;
   if( d->lz_decoder )
     { LZd_free( d->lz_decoder ); free( d->lz_decoder ); d->lz_decoder = 0; }
-  if( Rd_find_header( d->rdec ) ) d->seeking = false;
+  if( Rd_find_header( d->rdec, &skipped ) ) d->seeking = false;
   else
     {
     if( !d->rdec->at_stream_end ) d->seeking = true;
-    else { d->seeking = false; Rd_purge( d->rdec ); }
+    else { d->seeking = false; d->partial_in_size += Rd_purge( d->rdec ); }
     }
+  d->partial_in_size += skipped;
   d->lz_errno = LZ_ok;
   d->fatal = false;
   return 0;
@@ -406,38 +401,70 @@ int LZ_decompress_read( struct LZ_Decoder * const d,
   {
   int result;
   if( !verify_decoder( d ) || d->fatal ) return -1;
-  if( d->seeking ) return 0;
+  if( d->seeking || size < 0 ) return 0;
 
   if( d->lz_decoder && LZd_member_finished( d->lz_decoder ) )
     {
-    d->partial_in_size += d->rdec->member_position;
     d->partial_out_size += LZd_data_position( d->lz_decoder );
     LZd_free( d->lz_decoder ); free( d->lz_decoder ); d->lz_decoder = 0;
     }
   if( !d->lz_decoder )
     {
-    if( Rd_available_bytes( d->rdec ) < 5 + Fh_size )
+    int rd;
+    d->partial_in_size += d->rdec->member_position;
+    d->rdec->member_position = 0;
+    if( Rd_available_bytes( d->rdec ) < Fh_size + 5 &&
+        !d->rdec->at_stream_end ) return 0;
+    if( Rd_finished( d->rdec ) && !d->first_header ) return 0;
+    rd = Rd_read_data( d->rdec, d->member_header, Fh_size );
+    if( Rd_finished( d->rdec ) )		/* End Of File */
       {
-      if( !d->rdec->at_stream_end || Rd_finished( d->rdec ) ) return 0;
-      /* set position at EOF */
-      d->partial_in_size += Rd_available_bytes( d->rdec );
-      if( Rd_available_bytes( d->rdec ) <= Fh_size ||
-          !Rd_read_header( d->rdec, d->member_header ) )
-        d->lz_errno = LZ_header_error;
-      else
+      if( rd <= 0 || Fh_verify_prefix( d->member_header, rd ) )
         d->lz_errno = LZ_unexpected_eof;
+      else
+        d->lz_errno = LZ_header_error;
       d->fatal = true;
       return -1;
       }
-    if( !Rd_read_header( d->rdec, d->member_header ) )
+    if( !Fh_verify_magic( d->member_header ) )
       {
-      d->lz_errno = LZ_header_error;
+      /* unreading the header prevents sync_to_member from skipping a member
+         if leading garbage is shorter than a full header; "lgLZIP\x01\x0C" */
+      if( Rd_unread_data( d->rdec, rd ) )
+        {
+        if( d->first_header || !Fh_verify_corrupt( d->member_header ) )
+          d->lz_errno = LZ_header_error;
+        else
+          d->lz_errno = LZ_data_error;		/* corrupt header */
+        }
+      else
+        d->lz_errno = LZ_library_error;
+      d->fatal = true;
+      return -1;
+      }
+    if( !Fh_verify_version( d->member_header ) ||
+        !isvalid_ds( Fh_get_dictionary_size( d->member_header ) ) )
+      {
+      if( Rd_unread_data( d->rdec, 1 + !Fh_verify_version( d->member_header ) ) )
+        d->lz_errno = LZ_data_error;	/* bad version or bad dict size */
+      else
+        d->lz_errno = LZ_library_error;
+      d->fatal = true;
+      return -1;
+      }
+    d->first_header = false;
+    if( Rd_available_bytes( d->rdec ) < 5 )
+      {
+      /* set position at EOF */
+      d->rdec->member_position += Cb_used_bytes( &d->rdec->cb );
+      Cb_reset( &d->rdec->cb );
+      d->lz_errno = LZ_unexpected_eof;
       d->fatal = true;
       return -1;
       }
     d->lz_decoder = (struct LZ_decoder *)malloc( sizeof (struct LZ_decoder) );
     if( !d->lz_decoder || !LZd_init( d->lz_decoder, d->rdec,
-                          Fh_get_dictionary_size( d->member_header ) ) )
+                             Fh_get_dictionary_size( d->member_header ) ) )
       {					/* not enough free memory */
       if( d->lz_decoder )
         { LZd_free( d->lz_decoder ); free( d->lz_decoder ); d->lz_decoder = 0; }
@@ -445,11 +472,16 @@ int LZ_decompress_read( struct LZ_Decoder * const d,
       d->fatal = true;
       return -1;
       }
+    d->rdec->reload_pending = true;
     }
   result = LZd_decode_member( d->lz_decoder );
   if( result != 0 )
     {
-    if( result == 2 ) d->lz_errno = LZ_unexpected_eof;
+    if( result == 2 )			/* set position at EOF */
+      { d->rdec->member_position += Cb_used_bytes( &d->rdec->cb );
+        Cb_reset( &d->rdec->cb );
+        d->lz_errno = LZ_unexpected_eof; }
+    else if( result == 5 ) d->lz_errno = LZ_library_error;
     else d->lz_errno = LZ_data_error;
     d->fatal = true;
     return -1;
@@ -463,12 +495,15 @@ int LZ_decompress_write( struct LZ_Decoder * const d,
   {
   int result;
   if( !verify_decoder( d ) || d->fatal ) return -1;
+  if( size < 0 ) return 0;
 
   result = Rd_write_data( d->rdec, buffer, size );
   while( d->seeking )
     {
     int size2;
-    if( Rd_find_header( d->rdec ) ) d->seeking = false;
+    unsigned skipped = 0;
+    if( Rd_find_header( d->rdec, &skipped ) ) d->seeking = false;
+    d->partial_in_size += skipped;
     if( result >= size ) break;
     size2 = Rd_write_data( d->rdec, buffer + result, size - result );
     if( size2 > 0 ) result += size2;
@@ -539,18 +574,15 @@ unsigned long long LZ_decompress_data_position( struct LZ_Decoder * const d )
 
 unsigned long long LZ_decompress_member_position( struct LZ_Decoder * const d )
   {
-  if( verify_decoder( d ) && d->lz_decoder )
-    return d->rdec->member_position;
-  return 0;
+  if( !verify_decoder( d ) ) return 0;
+  return d->rdec->member_position;
   }
 
 
 unsigned long long LZ_decompress_total_in_size( struct LZ_Decoder * const d )
   {
   if( !verify_decoder( d ) ) return 0;
-  if( d->lz_decoder )
-    return d->partial_in_size + d->rdec->member_position;
-  return d->partial_in_size;
+  return d->partial_in_size + d->rdec->member_position;
   }
 
 
